--[[ Roster module to Handle the Raid Roster    ]]
--[[ tab widths set to 4 for optimal formatting ]]

local _, TriageTable = ...; 

local Triage = TriageTable.Triage
Triage.Dialog = Triage:NewModule("Dialog", "AceEvent-3.0", "AceBucket-3.0")
local Dialog = Triage.Dialog

local L = LibStub("AceLocale-3.0"):GetLocale("Triage");


-- The various state tables.
Dialog.heal = { }
Dialog.heal.labels = { }
Dialog.heal.unassigned = { }
Dialog.heal.assigned = { }
Dialog.dps = { }
Dialog.dps.labels = { }
Dialog.dps.unassigned = { }
Dialog.dps.assigned = { }
Dialog.tank = { }
Dialog.tank.labels = { }
Dialog.tank.unassigned = { }
Dialog.tank.assigned = { }
Dialog.other = { }
Dialog.other.labels = { }
Dialog.other.unassigned = { }
Dialog.other.assigned = { }

Dialog.role = { }
Dialog.active_tab = "HEALER"

Dialog.unassigned_slot_list = { }
Dialog.assignment_slot_list = { }

Dialog.alpha_raid = 0.9
Dialog.alpha_waiting = 0.3




function Dialog:Print( str )
	Triage.debugFrame:AddMessage( "|c933f9fffTriage:|r " .. str )
end



function Dialog:Debug( str, ... )
	Triage:Debug("|c933f9fffDialog|r - "..str , ... )
end



function Dialog:OnInitialize() 
	self:Debug( "OnInitialize" )

	PanelTemplates_SetNumTabs( TriageDialog, 4 );
	PanelTemplates_SetTab( TriageDialog, 1 );

	StaticPopupDialogs[ "TriageEditAssignment" ] = {
		text = L["Edit Assignment"],
		button1 = TEXT( SAVE ),
		button2 = TEXT( CANCEL ),
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		hasEditBox = 1,

		OnAccept = function( frame )
			self:Debug( "TriageEditAssignment:OnAccept() " .. frame:GetName() )
			local editbox = _G[ frame:GetName() .. "EditBox" ]
			self:ApplyAssignmentGroupEdit( editbox:GetText() )
		end,

		EditBoxOnEnterPressed = function( editbox )
			self:Debug( "TriageEditAssignment:EditBoxOnEnterPressed() " .. editbox:GetName() )
			self:ApplyAssignmentGroupEdit( editbox:GetText() )
			editbox:GetParent():Hide()
		end,

		EditBoxOnEscapePressed = function( frame )
			self:Debug( "TriageEditAssignment:EditBoxOnEscapePressed() " .. frame:GetName() )
			frame:GetParent():Hide()
		end,

		-- Set the edit box text to the label when box is shown
		OnShow = function( frame )
			self:Debug( "TriageEditAssignment:OnShow() " .. frame:GetName() )
			local editbox = _G[ frame:GetName() .. "EditBox" ]
			editbox:SetText( "" )
		end,

		-- Clear the edit box text when the box is hidden
		OnHide = function( frame )
			self:Debug( "TriageEditAssignment:OnHide() " .. frame:GetName() )
			local editbox = _G[ frame:GetName() .. "EditBox" ]
			editbox:SetText( "" )
		end,

	}

end



function Dialog:OnEnable()
	self:Debug( "OnEnable" )

	if not self.frame then
		self:CreateDialog()
	end

	self:RegisterMessage( "Triage_RosterUpdated", "RosterUpdated" )

	if oRA3 then
		self:Debug( "Registering for oRA3 Tanks" )
		oRA3.RegisterCallback( self, "OnTanksUpdated", "ORA3TanksUpdated" )
		self:ORA3TanksUpdated( "OnTanksUpdated", oRA3:GetSortedTanks() )
	end

	self:Debug( "OnEnable complete" )
end



function Dialog:OnLoad()
	self:Debug( "OnLoad" )
end



function Dialog:CreateDialog()
	self:Debug( "CreateDialog" )

	-- Sizes/counts of various elements
	self.available_size = 25
	self.group_count = 8
	self.group_size = 5

	-- Localize the XML items
	_G["TriageDialogAvailableLabel"]:SetText( L["Unassigned"] )
	for i = 1, self.available_size do
		local key = ( "TriageDialogAvailableSlot%dLabel" ):format( i )
		local value = "Empty"
		_G[key]:SetText( L[value] )
	end

	for i = 1, self.group_count do
		local key = ( "TriageDialogAssignmentGroup%dLabel" ):format( i )
		local value = ( "Assignment %d" ):format( i )
		_G[key]:SetText( L[value] )
		for k = 1, self.group_size do
			key = ( "TriageDialogAssignmentGroup%dSlot%dLabel" ):format( i, k )
			value = "Empty"
			_G[key]:SetText( L[value] )
		end
	end

	_G["TriageDialogChannelEditBoxLabel"]:SetText( L["Report Channel"] ) 

	_G["TriageDialogReport"]:SetText( L["Report"] )
	_G["TriageDialogTab1"]:SetText( L["Healing"] )
	_G["TriageDialogTab2"]:SetText( L["Damage"] )
	_G["TriageDialogTab3"]:SetText( L["Tanking"] )
	_G["TriageDialogTab4"]:SetText( L["Other"] )


	-- Create assignment groups tables
	for i = 1, self.group_count do
		self.heal.assigned[ i ] = { }
		self.dps.assigned[ i ] = { }
		self.tank.assigned[ i ] = { }
		self.other.assigned[ i ] = { }

		-- Save the destination slot names to the assignment group slot list	
		for k = 1, self.group_size do
			local key = ( "TriageDialogAssignmentGroup%dSlot%d" ):format( i, k )
			table.insert( self.assignment_slot_list, key )
		end
	end

	-- Save the available slot names to the unassigned slot list	
	for i = 1, self.available_size do
		local key = ( "TriageDialogAvailableSlot%d" ):format( i )
		table.insert( self.unassigned_slot_list, key )
	end

	-- Create the buttons
	local frame_level = TriageDialog:GetFrameLevel()
	self.available_buttons = { }
	for i = 1, self.available_size do
		local name = ( "TriagePlayerButton%d" ):format( i )
		local button = CreateFrame( "Button", name, TriageDialog, 
								"TriageButtonTemplate" )
		button:ClearAllPoints()
		button:SetFrameLevel( frame_level + 5 )
		button:SetMovable( true )
		button:RegisterForDrag( "LeftButton" )
		button:EnableMouse( true )
		button:SetClampedToScreen( true )
		button:Hide()
		self.available_buttons[ i ] = button
		self.next_available_button = i
	end

	self:StoreActiveLabels( "HEALER" )
	self:StoreActiveLabels( "DAMAGER" )
	self:StoreActiveLabels( "TANK" )
	self:StoreActiveLabels( "OTHER" )

	self.frame = TriageDialog
	self:Debug( "CreateDialog complete" )
end



function Dialog:RosterUpdated( event )
	self:Debug( "RosterUpdated" )
	local raid_size, raid_roster, waiting_roster, class_table, role_table =
		Triage.Roster:GetRaidRoster()

	if Triage.debug then
		raid_list = ( "Raid size: %d -- Inside:" ):format( raid_size )
		for i = 1, #raid_roster do
    		raid_list = raid_list.." "..raid_roster[i]
		end
		raid_list = raid_list.." -- Outside:"
		for i = 1, #waiting_roster do
    		raid_list = raid_list.." "..waiting_roster[i]
		end
		self:Debug( raid_list )
	end

	for i = 1, #raid_roster do
		local name = raid_roster[ i ]
		local role = role_table[ name ]
		self:PopulateTables( name, role )
		self:Populate( name, "OTHER" )		-- add all in raid to "OTHER"
	end

	for i = 1, #waiting_roster do
		local name = waiting_roster[ i ]
		local role = role_table[ name ]
		self:PopulateTables( name, role )
	end


	
	self:PruneTables( raid_roster, waiting_roster, "HEALER" )
	self:PruneTables( raid_roster, waiting_roster, "DAMAGER" )
	self:PruneTables( raid_roster, waiting_roster, "TANK" )
	self:PruneTables( raid_roster, waiting_roster, "OTHER" )

	self:BuildActiveTab( raid_roster, waiting_roster, class_table )
end



function Dialog:PopulateTables( name, role )

	local previous_role = self.role[ name ]
	if previous_role then
		if previous_role ~= role then
			local prev_unassigned, prev_assigned = 
				self:GetActiveTables( previous_role )
			self:RemovePlayerFromTables( prev_unassigned, prev_assigned, name )
			self:Debug( "Role changed for " .. name .. " - " .. previous_role .. " to " .. role )
		end
	end
	self.role[ name ] = role

	if role ~= "NONE" then
		self:Populate( name, role )
	end
end



function Dialog:Populate( name, role )
	
	local unassigned, assigned = self:GetActiveTables( role )
	local found = false
	if tContains( unassigned, name ) then
		self:Debug( name .. " found in unassigned for " .. role )
		found = true
	else
		for i, v in ipairs( assigned ) do
			if tContains( v, name ) then
				self:Debug( name .. " found in assigned for " .. role )
				found = true
				break
			end
		end
	end

	if not found then
		if not tContains( unassigned, name ) then
			table.insert( unassigned, name )
			self:Debug( name .. " added to unassigned for " .. role )
		end
	end

end


function Dialog:PruneTables( raid_roster, waiting_roster, role )
	self:Debug( "PruneTables " .. role )

	local unassigned, assigned = self:GetActiveTables( role )

	local to_prune = { }
	for i = 1, #unassigned do 
		local name = unassigned[ i ]
		if tContains( raid_roster, name ) or
			tContains( waiting_roster, name ) then
		else
			self:Debug( "Adding " .. name .. " in unassigned to be pruned" )
			table.insert( to_prune, name )
		end
	end

	for i = 1, #assigned do 
		local group = assigned[ i ]
		for k = 1, #group do
			local name = group[ k ]
			if tContains( raid_roster, name ) or
				tContains( waiting_roster, name ) then
			else
				self:Debug( ( "Adding " .. name .. " in group %d to be pruned" ):format( k ) )
				table.insert( to_prune, name )
			end
		end
	end

	for i = 1, #to_prune do
		local name = to_prune[ i ]
		self:RemovePlayerFromTables( unassigned, assigned, name )
	end

end



function Dialog:BuildActiveTab( raid_roster, waiting_roster, class_table )
	self:Debug( "Building active tab: " .. self.active_tab )

	local unassigned, assigned = self:GetActiveTables( self.active_tab )	

	self:ApplyActiveLabels()

	local editbox = _G["TriageDialogChannelEditBox"]
	editbox:SetText( Triage:GetReportChannel( self.active_tab ) )
	editbox:SetScript( "OnTextChanged", function(self)
		Triage:SetActiveReportChannel( nil, self:GetText() ) 
		end )

	self.next_available_button = 25
	for i = 1, #self.available_buttons do
		local button = self.available_buttons[ i ]
		button:Hide()
	end

	local raid_slot = 1
	local waiting_slot = 25
	for i = 1, #unassigned do
		local name = unassigned[ i ]
		self:Debug( "** Unassigned player " .. name )

		local in_raid = false
		local waiting = false
		if tContains( raid_roster, name ) then
			in_raid = true
		elseif tContains( waiting_roster, name ) then
			waiting = true
		end

		if in_raid or waiting then
			local slot
			if in_raid then
				slot = ( "TriageDialogAvailableSlot%d" ):format( raid_slot )
				raid_slot = raid_slot + 1
			else
				slot = ( "TriageDialogAvailableSlot%d" ):format( waiting_slot )
				waiting_slot = waiting_slot - 1
			end

			local class = strupper( class_table[ name ] )
			self:Debug( name .. " is " .. class ) 

			self:PrepareButton( slot, name, class, in_raid )
		else
			self:Debug( name .." not found in raid or waiting rosters" )
			self:RemovePlayerFromTables( unassigned, assigned, name )
		end
	end

	for i = 1, #assigned do 
		local group = assigned[ i ]
		for k = 1, #group do
			local name = group[ k ]
			self:Debug( ( "Group %d, Name %d is %s" ):format( i, k, name ) )

			local in_raid = false
			local waiting = false
			if tContains( raid_roster, name ) then
				in_raid = true
			elseif tContains( waiting_roster, name ) then
				waiting = true
			end

			if in_raid or waiting then
				local slot = ( "TriageDialogAssignmentGroup%dSlot%d" ):format( i, k )
				local class = strupper( class_table[ name ] )
				self:Debug( name .. " is " .. class ) 

				self:PrepareButton( slot, name, class, in_raid )
			else
				self:Debug( name .." not found in raid or waiting rosters" )
				self:RemovePlayerFromTables( unassigned, assigned, name )
			end
		end
	end

end



function Dialog:ApplyActiveLabels() 
	local unassigned, assigned_labels_table =
		self:GetActiveLabels( self.active_tab )

	local label = _G[ "TriageDialogAvailableLabel" ]
	label:SetText( unassigned )

	for i = 1, self.group_count do
		local label_text = assigned_labels_table[ i ]
		local label_name = ( "TriageDialogAssignmentGroup%dLabel" ):format( i )
		label = _G[ label_name ]
		if label then
			label:SetText( label_text )
		else
			self:Debug( "******* NO LABEL FOR " .. label_name )
		end
	end
end



function Dialog:StoreActiveLabels( role )
	
	local label = _G[ "TriageDialogAvailableLabel" ]
	local value = label:GetText()
	if role == "HEALER" then
		self.heal.unassigned_label = value
	elseif role == "DAMAGER" then
		self.dps.unassigned_label = value
	elseif role == "TANK" then
		self.tank.unassigned_label = value
	elseif role == "OTHER" then
		self.other.unassigned_label = value
	else
		self:Debug( "StoreActiveLabels() - invalid role " .. role )
		return
	end

	local _, assigned_labels_table = self:GetActiveLabels( role )
	table.wipe( assigned_labels_table )
	for i = 1, self.group_count do
		local label_name = ( "TriageDialogAssignmentGroup%dLabel" ):format( i )
		label = _G[ label_name ]
		table.insert( assigned_labels_table, label:GetText() )
	end
end



function Dialog:PrepareButton( slot, name, class, in_raid )
	local button = self.available_buttons[ self.next_available_button ]
	self.next_available_button = self.next_available_button - 1
	button:ClearAllPoints()
	button:SetPoint( "TOPLEFT", slot, "TOPLEFT" )

	local label = _G[button:GetName().."Label"]
	label:SetText( name )

	local color = RAID_CLASS_COLORS[class]
	local button_color = _G[button:GetName().."Color"]
	if in_raid then
		button_color:SetTexture( color.r, color.g, color.b, self.alpha_raid )
	else
		button_color:SetTexture( color.r, color.g, color.b, self.alpha_waiting )
	end

	button:Show()
end



function Dialog:GetActiveTables( role )

	local unassigned, assigned

	if role == "HEALER" then
		unassigned = self.heal.unassigned
		assigned = self.heal.assigned
	elseif role == "DAMAGER" then
		unassigned = self.dps.unassigned
		assigned = self.dps.assigned
	elseif role == "TANK" then
		unassigned = self.tank.unassigned
		assigned = self.tank.assigned
	else
		unassigned = self.other.unassigned
		assigned = self.other.assigned
	end

	return unassigned, assigned
end



function Dialog:GetActiveLabels( role )

	local unassigned, label_table

	if role == "HEALER" then
		unassigned = self.heal.unassigned_label
		label_table = self.heal.labels
	elseif role == "DAMAGER" then
		unassigned = self.dps.unassigned_label
		label_table = self.dps.labels
	elseif role == "TANK" then
		unassigned = self.tank.unassigned_label
		label_table = self.tank.labels
	elseif role == "OTHER" then
		unassigned = self.other.unassigned_label
		label_table = self.other.labels
	else
		self:Debug( "GetActiveLabels() passed invalid role " .. role )
		return
	end

	return unassigned, label_table
end



function Dialog:ORA3TanksUpdated( event, tanks )
	self.ora3_tanks = tanks
	local tank_list = "oRA3 Tanks:"
	for i = 1, #tanks do
		tank_list = tank_list.." "..tanks[ i ]
	end
	Dialog:Debug( tank_list )
end



function Dialog:EditAssignmentGroup( button )
	self:Debug( "EditAssignmentGroup ".. button:GetName() )
	self.button_being_edited = button
	self.label_being_edited = _G[button:GetName() .. "Label"]
	StaticPopup_Show( "TriageEditAssignment", button:GetID() )
end



function Dialog:ApplyAssignmentGroupEdit( assignment )
	self.label_being_edited:SetText( assignment )
	self:StoreActiveLabels( self.active_tab )

 	self.button_being_edited = nil
	self.label_being_edited = nil
end



function Dialog:CreateAvailable( parent )
	self:Debug( "CreateAvailable" )

	local frame = CreateFrame( "Frame", "TriageAvailable", parent )

	frame:SetWidth( 100 )
	frame:SetHeight( 261 )

	frame:SetPoint( "TOPLEFT", parent, "TOPLEFT", 14, -45 )

	frame:SetBackdrop{
		bgFile="Interface\\DialogFrame\\UI-DialogBox-Background" ,
--		edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
		tileSize = 32, edgeSize = 32,
	}

	frame.texture = frame:CreateTexture( nil, "BORDER" )
	frame.texture:SetPoint( "TOPLEFT", frame, "TOPLEFT", 4, -4 )
	frame.texture:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4 )
	frame.texture:SetBlendMode( "DISABLE" )
--	frame.texture:SetGradientAlpha( "VERTICAL", .1, .1, .1, 0, .2, .2, .2, 0.5 )
	frame:EnableMouse( true )

	frame.slots = { }
	for index, slotname in ipairs( self.available ) do
		local slot = self:CreateAvailableSlot( frame, index, slotname )
		frame.slots[ index ] = slot
	end

	parent.availableFrame = frame
end




function Dialog:Dialog()
	self:Debug( "Dialog" )
	if self.frame then
		if self.frame:IsShown() then
			self:HideDialog()
		else
			self:ShowDialog()
		end
	end
end



function Dialog:ShowDialog()
	self:Debug( "ShowDialog" )
	if Triage.inCombat then
		self:Debug( "  in combat, refusing to display" );
		return
	end
	if self.frame then
		self.frame:Show()
	else 
		 TriageDialog:Show()
	end
	
end



function Dialog:HideDialog()
	self:Debug( "HideDialog" )
	if self.frame then
		self.frame:Hide()
	end
end



function Dialog:PlayerButtonOnLoad( button )
	self:Debug( "PlayerButtonOnLoad: "..button:GetName() )
end



function Dialog:PlayerButtonOnClick( button )
	self:Debug( "PlayerButtonOnClick: "..button:GetName() )
end



function Dialog:PlayerButtonOnDragStart( button )
	self:Debug( "PlayerButtonOnDragStart: "..button:GetName() )
	button:ClearAllPoints()

	local moving_level = button:GetParent():GetFrameLevel() + 40;
	button:SetFrameLevel( moving_level )
	button:StartMoving()
end




function Dialog:PlayerButtonOnDragStop( button )
	self:Debug( "PlayerButtonOnDragStop: "..button:GetName() )

	local stopped_level = button:GetParent():GetFrameLevel() + 5
	button:SetFrameLevel( stopped_level )
	button:StopMovingOrSizing()

	local unassigned, assigned = self:GetActiveTables( self.active_tab )

	local found = false

	-- check the assignment groups first
	for i, slot_name in ipairs( self.assignment_slot_list ) do
		local slot = _G[slot_name]
		if MouseIsOver( slot ) then
			local player = _G[button:GetName() .. "Label"]:GetText()
			self:Debug( "Dropping " .. button:GetName() .. " - " .. player .. " over " .. slot_name )

			-- Look for something in the format of ".*Group%dSlot%d"
			local _, _, group_str, slot_str = 
				string.find( slot_name, ".+(%d+).+(%d+)" )
			--self:Debug( ( "Group %d, Slot %d" ):format( group_num, slot_num ) )
			local group_num = tonumber( group_str )
			self:RemovePlayerFromTables( unassigned, assigned, player )

			local destination = assigned[ group_num ]
			table.insert( destination, player )
			table.sort( destination )
			found = true
		end
	end

	-- check the unassigned list if not found yet
	if not found then
		for i, slot_name in ipairs( self.unassigned_slot_list ) do
			local slot = _G[slot_name]
			if MouseIsOver( slot ) then
				local player = _G[button:GetName() .. "Label"]:GetText()
				self:Debug( "Dropping " .. button:GetName() .. " - " .. player .. " over " .. slot_name )

				-- Look for something in the format of ".*Slot%d"
				local _, _, slot_num = string.find( slot_name, ".+(%d+)" )
				--self:Debug( ( "AvailableSlot %d" ):format( slot_num ) )

				self:RemovePlayerFromTables( unassigned, assigned, player )
				table.insert( unassigned, player )
				table.sort( unassigned )
				found = true
			end
		end
	end

	self:RosterUpdated()
end



function Dialog:RemovePlayerFromTables( unassigned, assigned, name )
	self:Debug( "RemovePlayerFromTables: "..name )	

	self:RemoveFromTable( unassigned, name )

	for group_num = 1, #assigned do
		local group = assigned[group_num]
		self:RemoveFromTable( group, name )
	end
end



function Dialog:RemoveFromTable( table, item )

	local limit = #table
	for i = 1, limit do
		if table[ i ] == item then
			table[ i ] = table[ limit ]
			table[ limit ] = nil
			return
		end
	end

end



function Dialog:ActivateTab( tab, id )
	self:Debug( "ActivateTab " .. tab .. ", " .. id )

	if tab == "HEALER" then
		self.active_tab = "HEALER"
	elseif tab == "DAMAGER" then
		self.active_tab = "DAMAGER"
	elseif tab == "TANK" then
		self.active_tab = "TANK"
	elseif tab == "OTHER" then
		self.active_tab = "OTHER"
	else
		self:Print( "No tab named " .. tab )
		return
	end

	local editbox = _G["TriageDialogChannelEditBox"]
	local channel = Triage:GetReportChannel( self.active_tab )
	editbox:SetText( channel )

	self:Debug( "Sending Triage_RosterUpdated from ActivateTab()" )
	Triage.Roster:SendMessage( "Triage_RosterUpdated" )

	PanelTemplates_SetTab( self.frame, id );
end



function Dialog:ReportActive()
	self:Debug( "ReportActive" )
	if self.active_tab == "HEALER" then
		self:ReportHealers()
	elseif self.active_tab == "DAMAGER" then
		self:ReportDamagers()
	elseif self.active_tab == "TANK" then
		self:ReportTanks()
	elseif self.active_tab == "OTHER" then
		self:ReportOther()
	else
		self:Print( "No active tab" )
	end

end



function Dialog:ReportHealers()
	self:Debug( "ReportHealers" )
	local unassigned, assigned = self:GetActiveTables( "HEALER" )
	Triage:SendOutput( "HEALER", L["Triage"] .. " " .. L["Healing assignments"] )
	self:BuildReport( unassigned, assigned, "HEALER", true )
end



function Dialog:ReportDamagers()
	self:Debug( "ReportDamagers" )
	local unassigned, assigned = self:GetActiveTables( "DAMAGER" )
	Triage:SendOutput( "DAMAGER", L["Triage"] .. " " .. L["DPS assignments"] )
	self:BuildReport( unassigned, assigned, "DAMAGER", true )
end



function Dialog:ReportTanks()
	self:Debug( "ReportTanks" )
	local unassigned, assigned = self:GetActiveTables( "TANK" )
	Triage:SendOutput( "TANK", L["Triage"] .. " " .. L["Tanking assignments"] )
	self:BuildReport( unassigned, assigned, "TANK", true )
end



function Dialog:ReportOther()
	self:Debug( "ReportOther" )
	local unassigned, assigned = self:GetActiveTables( "OTHER" )
	Triage:SendOutput( "OTHER", L["Triage"] .. " " .. L["Other assignments"] )
	self:BuildReport( unassigned, assigned, "OTHER", false )
end



function Dialog:BuildReport( unassigned, assigned, role, report_unassigned )

	local _, raid_roster, waiting_roster = Triage.Roster:GetRaidRoster()
	local unassigned_label, label_table = self:GetActiveLabels( role )
	
	for i = 1, #assigned do
		local group = assigned[ i ]
		if #group > 0 then
			local output = label_table[ i ] .. ":"
			for k = 1, #group do
				local name = group[ k ]
				if tContains( raid_roster, name ) then
					output = output .. " " .. name
					Triage:SendWhisper( name, role, label_table[ i ], true )
				elseif tContains( waiting_roster, name ) then
					output = output .. " **" .. strupper( name ) .. "**"
					Triage:SendWhisper( name, role, label_table[ i ], false )
				end
			end
			Triage:SendOutput( role, output )
		end
	end

	if #unassigned > 0 and report_unassigned then
		local output = unassigned_label .. ":"
		local unassigned_in_raid = false
		for i = 1, #unassigned do
			local name = unassigned[ i ]
			if tContains( raid_roster, name ) then
				output = output .. " " .. name
				unassigned_in_raid = true
				Triage:SendWhisper( name, role, unassigned_label, true )
			end
		end
		if unassigned_in_raid then
			Triage:SendOutput( role, output )
		end
	end

end